/* RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and that
no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive

                    (703) 742-4795
                    markk@internic.net

File: rwhios.h   Header file for all RWhois client files.
Developed by: Scott Williamson
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <errno.h>
#include <ctype.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif /* HAVE_FCNTL_H */
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif /* HAVE_SYS_FILE_H */
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#ifdef TIME_WITH_SYS_TIME
#include <time.h>
#endif /* TIME_WITH_SYS_TIME */
#endif /* HAVE_SYS_TIME_H */

/* #define  DEBUG */

#define NullCH    '\0'   /* the NULL Character */

#ifndef bzero
#define bzero(a,b) memset(a,NullCH,b)
#endif
#ifndef bcopy
#define bcopy(a,b,c) memcpy(b,a,c)
#endif
#ifndef index
#define index strchr
#endif
#ifndef rindex
#define rindex strrchr
#endif

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 256 /*RFC 1123 */
#endif

#define SERVICE     "rwhois"
#define PROTOCOL    "tcp"
#define NORMPORT    4321
#define MAXSERVERS  90
#define MAXBUFFER   128

#define SERDEF1     "rwhois.internic.net:4321:rwhois"
#define SERDEF2     "rs.internic.net:43:whois"
#define SERDEF3     "nic.ddn.mil:43:whois"

#define FILENAMEMASK    "//tmp//rwhois.XXXXXX"
#define MAXBUFF 256
#define MAXLINE 81

#define DEFAULTSHOT 0
#define SINGLESHOT  1
#define MULTISHOT   2

#define TRAIL_RESET 0
#define TRAIL_ADD   1
#define TRAIL_COMP  2
#define TRAIL_LIST  3

#define REF_PRESET      0x00
#define REF_CMDLINE     0x01
#define REF_CONFFILE    0x02
#define REF_SEEALSO     0x04
#define REF_REFERRAL    0x10
#define REF_PUNT        0x20
#define REF_UNKNOWN     0x40

#define PANNEL_COUNT    19

#define CP_QUERY    0
#define CP_PROMPT   1
#define CP_HOLDCONNECT  2
#define CP_LIMIT    3
#define CP_SOA  4
#define CP_CACHE    5
#define CP_XFER 6
#define CP_FORWARD  7
#define CP_SCHEMA   8
#define CP_STATUS   9
#define CP_LOAD 10
#define CP_DIRECTIVE    11
#define CP_OBJECT   12
#define CP_REFERRAL 13
#define CP_SEEALSO  14
#define CP_PRECMD   15
#define CP_DISPLAY  16
#define CP_REGISTER 17
#define CP_DEFINE   18

#define SET_NONE    0x00
#define SET_SEND    0x01
#define SET_WAIT    0x02
#define SET_FORCE   0x04
#define SET_EDIT    0x10
#define SET_ON      0x20
#define SET_OFF     0x40
#define SET_ALWAYS  0x80

#define LIMIT_SET   0
#define LIMIT_SEND  1
#define LIMIT_RESEND    2
#define LIMIT_CURRENT   3
#define LIMIT_DEFAULT   4

#define CACHE_OFF   0
#define CACHE_ON    1

#define FORWARD_OFF 0
#define FORWARD_ON  1

#define MODE_NONE   0
#define MODE_INFO   1
#define MODE_MIME   2

#define REG_NONE    0
#define REG_ADD     1
#define REG_MOD 2
#define REG_DEL 3

#define WAIT_NO -1
#define WAIT_NULL   0
#define WAIT_IDYES  1
#define WAIT_IDNO   2
#define WAIT_OK 3
#define WAIT_ERROR  4
#define WAIT_REFERRAL   5
#define WAIT_SEEALSO    6
#define WAIT_EDIT   7
#define WAIT_CONTINUE   8
#define WAIT_SOMETHING  99

#define DISPLAY_NONE    0
#define DISPLAY_MIME    1
#define DISPLAY_URL     2

/* used during contact of server */
#define CONTGOOD    0
#define CONTBADADD  1   /* Bad address resol */
#define CONTBADSOCK 2   /* Bad socket action  */
#define CONTBADCON  3   /* Bad connect */
#define CONTBADMODE 4   /* Bad mode change */

/* file status flags */
#define FILECLOSED  0
#define FILEOPEN    2
#define FILEEMPLY   4
#define FILEHASDATA 8

#define TIMEOUT -1

#define ERROR   -1

#define CONFPATH    "/etc" /* system wide config file */

#define SPEC_VERSION    "1.0"
#define IMP_VERSION "1.0BETA4"

#ifndef TRUE
#define TRUE    1
#endif
#ifndef FALSE
#define FALSE   0
#endif

#ifndef GOOD
#define GOOD    0
#endif
#ifndef BAD
#define BAD 1
#endif

#ifndef ON
#define ON  1
#endif
#ifndef OFF
#define OFF     0
#endif

#define QUIET   0  /* no status printing */
#define WHISPER 1 /* bare minimum status printing */
#define TALK    2 /* constant status update */
#define SHOUT   3  /* detailed diagnostic update */

#define PRESET_SERVERS 2

#define DEF_WAITTIME    5
#define DEF_MAXWAIT 30
#define DEF_PORT    NORMPORT
#define DEF_LIMIT   20
#define DEF_LOAD    0

#define TYPE_UNKNOWN    0x00
#define TYPE_RWHOIS     0x01
#define TYPE_WHOIS      0x02
#define TYPE_OTHER      0x03
#define TYPE_WHOISPLUSPLUS  0x04

#define STATUS_QUERYSENT    0x10

/* Server status discovered using the -status directive */

struct STATUS {
    int limit;
    double load;
    int cache;
    int holdconnect;
    int forward;
    int authority;
    int cached;

};

/* Detail on each server */
struct SERVER {
    int server_id;
    char *server;
    int sh;
    int port;
    int reftype;
    int type_status;    /* type of server and current status */
    char *referral;
    char *query;
    char *filename;
    char *bkupname;
    int filestatus;
    FILE *fp;
    FILE *bkfp;
    time_t  start_time;     /* strat connection time */
    time_t  con_time;       /* connection time */
    time_t  ans_time;       /* answer time */
    struct STATUS server_status;
    struct SERVER *next_server;
    struct SERVER *prev_server;
};


#define SER_ADD         0 /* add to end of list */
#define SER_MODIFY      1 /* modify  current */
#define SER_DELETE      2 /* delete  current */
#define SER_INSERT      3 /* insert  after current*/
#define SER_CURRNEXT    4 /* move current to next */
#define SER_CURRPREV    5 /* move current to previous */
#define SER_CURRGET     6 /* get current */
#define SER_CURRHEAD    7 /* current move to head */
#define SER_MARK        8 /* set marker position to current */
#define SER_MARKHEAD    9   /* marker to head */
#define SER_MARKNEXT    10  /* marker to next */
#define SER_MARKPREV    11  /* marker to previous */
#define SER_MARKGET     12  /* get marker */
#define SER_INIT    13 /* initialize */
#define SER_FIND    14 /* find server (name only) in list */
#define SER_COMPARE 15      /* is server already on list */
#define SER_STARTTIME   16
#define SER_CONTIME 17
#define SER_ANSTIME 18
#define SER_ERASE   19  /* erase from next to end */
#define SER_REWIND      20  /* rewind the server list */

/* server maintenance macros */
#ifdef __STDC__
struct SERVER *server_main( int, struct SERVER *);
#else
struct SERVER *server_main();
#endif

#define SERVERHEAD  server_main( SER_CURRHEAD, (struct SERVER *) NULL)
#define SERVERMARK  server_main( SER_MARK, (struct SERVER *)  NULL)
#define SERVERMARKHEAD  server_main( SER_MARKHEAD,(struct SERVER *)  NULL)
#define SERVERMARKGET   server_main( SER_MARKGET,(struct SERVER *)  NULL)
#define SERVERMARKNEXT  server_main( SER_MARKNEXT,(struct SERVER *)  NULL)
#define SERVERMARKPREV  server_main( SER_MARKPREV,(struct SERVER *)  NULL)
#define SERVERINIT      server_main( SER_INIT, (struct SERVER *) NULL)
#define SERVERCURRGET   server_main( SER_CURRGET, (struct SERVER *) NULL)
#define SERVERCURRNEXT  server_main( SER_CURRNEXT, (struct SERVER *) NULL)
#define SERVERCURRPREV  server_main( SER_CURRPREV, (struct SERVER *) NULL)
#define SERVERSTARTTIME server_main( SER_STARTTIME, (struct SERVER *) NULL)
#define SERVERCONTIME   server_main( SER_CONTIME, (struct SERVER *) NULL)
#define SERVERANSTIME   server_main( SER_ANSTIME, (struct SERVER *) NULL)
#define SERVERERASE     server_main( SER_ERASE, (struct SERVER *) NULL)
#define SERVERREWIND    server_main( SER_REWIND, (struct SERVER *) NULL)

/* used in function list_server()  */

#define LISTALL      0  /* list all servers */
#define LISTBACK     1  /* list current back [already been] */
#define LISTFORWARD  2  /* list current forward [ to go ] */



#ifdef MAIN

int current_referral;

char *cmdname;
int shots;  /* single or multi shot referral */
int verbose;    /* bug the user with info or not */
int referral = PRESET_SERVERS;
char *confdir;
char *cachedir;
time_t start_time, con_time, ans_time;
int default_wait;
int default_maxwait;
int default_load;
int default_port;
int default_cache;
int default_forward;
int default_holdconnect;
int default_display;
int default_lcache;
short control_pannel[PANNEL_COUNT];
int extreferral, extsee_also;
char *user;
char *authstring;
char *autharea;
char *xfer_object;
char *object_name;
char *schema_object;
char *directive_name;
char *define_name;
int forcing;
int authority;
int current_modes;
int to_file;
int goodedit;
char file_name[30];
FILE *f_out;
char *mimepath, *urlpath, *editpath;
int mime, url;
int err_num;
char *precmd;
int edit;
int display_cap;
int info_on;
int punt_referral;
char *regfile;
int regtype;
int reg_only;


char *dirstrings[PANNEL_COUNT] = 
{
    "QUERY",
    "PROMPT",
    "HOLDCONNECT",
    "LIMIT",
    "SOA",
    "CACHE",
    "XFER",
    "FORWARD",
    "SCHEMA",
    "STATUS",
    "LOAD",
    "DIRECTIVE",
    "OBJECT",
    "REFERRAL",
    "SEEALSO",
    "PRECMD",
    "DISPLAY",
    "REGISTER",
    "DEFINE"
};

int just_show_config;
int preserve_temp;

#else
#ifdef __STDC__
extern char * safecalloc(int,int);
#else
extern char * safecalloc();
#endif
extern int current_referral;

extern char *cmdname;
extern int shots;  /* single or multi shot referral */
extern int verbose; /* bug the user with info or not */
extern int referral;
extern char *confdir;
extern char *cachedir;
extern time_t start_time, con_time, ans_time;
extern int default_wait;
extern int default_maxwait;
extern int default_load;
extern int default_port;
extern int default_cache;
extern int default_forward;
extern int default_holdconnect;
extern int default_display;
extern int default_lcache;
extern short control_pannel[PANNEL_COUNT];
extern int extreferral, extsee_also, start_over;
extern char *user;
extern char *authstring;
extern char *autharea;
extern char *xfer_object;
extern char *object_name;
extern char *schema_object;
extern char *directive_name;
extern char *define_name;
extern int forcing;
extern int authority;
extern int current_modes;

extern int to_file;
extern int goodedit;
extern char file_name[30];
extern FILE *f_out;
extern char *mimepath, *urlpath, *editpath;
extern int mime, url;
extern int err_num;
extern char *precmd;
extern int edit;
extern int display_cap;
extern int info_on;
extern int punt_referral;
extern char *regfile;
extern int regtype;
extern int reg_only;

extern char *dirstrings[PANNEL_COUNT];

extern int just_show_config; 
extern int preserve_temp;

#endif

