/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "commands.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include "defines.h"
#include "misc.h"
#include "client_msgs.h"

/* local statics */
static command_struct *commands = (command_struct *)NULL;

/* --------------------- LOCAL FUNCTIONS --------------------------- */

/* Old-style parse wants command of the form "<char><char>text" where
   char is "!" for handle-only, "#" for dump record, etc. */
static char *
do_magic(s, query_item, got_magic)
  char              *s;
  query_item_struct *query_item;
  int               *got_magic;
{
  char              c;
  command_struct    *cur = commands;

  if (s == NULL)
  {
    return (s);
  }

  c = *s;

  while (cur != NULL)
  {
    if (cur->magic != NULL && c ==  cur->magic[0])
    {
      switch (cur->type)
      {
      case COMMAND_DISPLAY:
        query_item->display = NEW_STRING(cur->command);
        s++;
        break;
      case COMMAND_QUERY: /* only one so far! */
        s++;
        query_item->attribute = NEW_STRING("handle");
        break;
      default:
        print_error(UNIDENT_ERROR, "command type %d has a magic!", cur->type);
        break;
      }

      *got_magic = TRUE;
      return (s);
    }

    cur = cur->nxt;
  }

  *got_magic = FALSE;

  return s;
}

/* --------------------- PUBLIC FUNCTIONS -------------------------- */

/* Parses the query string for a command */
void
parse_out_commands(query_item)
  query_item_struct *query_item;
{
  char  *t;
  char  *cp;
  char  *next;
  char  word[MAX_LINE];
  int   got_magic = TRUE;
  int   got_command = TRUE;

  t = cp = NEW_STRING(query_item->value);
  
  while (got_magic || (got_command && strlen(cp)))
  {
    /* first check for magic strings */
    if (!(cp = do_magic(cp, query_item, &got_magic)))
    {
      break;
    }
    next = get_word(cp, word);

    /* then check for other strings */
    if (do_commands (word, next, query_item, FALSE))
    {
      cp = next;
    }
    else
    {
      got_command = FALSE;
    }
  }
  strcpy (query_item->value, cp);

  free(t);
}


/* Checks the string against defined chars.  Returns TRUE if found,
      FALSE if not */
int
do_commands(word, next, query_item, directive_only)
  char              *word;
  char              *next;
  query_item_struct *query_item;
  int               directive_only;
{
  int               length;
  command_struct    *cur = commands;

  length = strlen(word);
  while (cur != NULL)
  {

    /* if in directive only mode, only get back command directives */
    if (directive_only && (cur->type != COMMAND_DIRECTIVE &&
                           cur->type != COMMAND_X_DIRECTIVE &&
                           cur->type != COMMAND_DISABLED_DIR));

    /* if not in directive only mode, don't look at directives */
    else if (!directive_only && cur->type == COMMAND_DIRECTIVE);

    /* else make sure that command is right length and has arg required */
    else if (cur->command != NULL && length >= cur->len &&
             STRN_EQ(word, cur->command, cur->len) &&
             ((cur->arg_required && strlen(next)) ||
              !cur->arg_required))
    {
      switch (cur->type)
      {
      case COMMAND_DISPLAY:
        query_item->display = NEW_STRING (cur->command);
        break;
      case COMMAND_QUERY:
        printf("do_commands: need to do this\n");
        break;
      case COMMAND_DIRECTIVE:
      case COMMAND_X_DIRECTIVE:
      case COMMAND_SPECIAL:
      case COMMAND_DISABLED_DIR:
        query_item->directive = NEW_STRING (cur->command);
        query_item->function = cur->function;
        if (cur->program != NULL) 
          query_item->program = NEW_STRING (cur->program);
        break;
      case COMMAND_OBJECT:
        query_item->object = cur->object;
        break;
      default:
        break;
      }
      return (TRUE);
    }
    cur = cur->nxt;
  }
  return (FALSE);
}

/* appends the command to the list */
void
append_command(command, len, magic, description, arg_required, type, object, 
               function, program)
  char          *command;
  int           len;
  char          *magic;
  char          *description;
  int           arg_required;
  command_type  type;
  object_struct *object;
  int           (*function)();
  char          *program;
{
  command_struct    *cur;
  command_struct    *new;

  new = NEW_COMMAND;
  
  if (command != NULL)
  {
    new->command = NEW_STRING(command);
  }
  new->len = len;
  if (magic != NULL)
  {
    new->magic =  NEW_STRING(magic);
  }
  if (description != NULL)
  {
    new->description =  NEW_STRING(description);
  }
  new->arg_required = arg_required;
  new->type = type;
  new->object = object;
  new->function = function;
  if (program != NULL)
  {
    new->program = NEW_STRING (program);
  }
  cur = commands;
  if (cur == NULL)
  {
    commands=new;
  }
  else
  {
    while (cur->nxt != NULL)
    {
      cur = cur->nxt;
    }
    cur->nxt = new;
  }
}

/* returns a object pointer if find command, if not found returns NULL */
object_struct *
return_object_by_command(str)
  char *str;

{
  int            length;
  command_struct *cur = commands;

  if (!str) return NULL;
  
  length = strlen(str);
  while (cur != NULL)
  {
    if (cur->type == COMMAND_OBJECT && cur->command != NULL && 
        length >= cur->len && strncasecmp (str, cur->command, cur->len) == 0)
    {
      return (cur->object);
    }

    cur = cur->nxt;
  }
  
  return NULL;
}


/* returns a pointer to command based on object, if not found returns
     NULL */
command_struct *
return_commands_by_object(object_str, reset)
  char  *object_str;
  int   reset;
{
  static command_struct *cur    = NULL;
  command_struct        *retval = NULL;

  if (reset)
  {
    cur = commands;
  }

  while (cur != NULL && retval == NULL)
  {
    if (cur->type == COMMAND_OBJECT && cur->command != NULL && 
        strcasecmp ( cur->object->name, object_str) == 0)
    {
      retval = cur;
    }

    cur = cur->nxt;
  }

  return (retval);
}

command_struct *
return_all_commands()
{
  return(commands);
}
