/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "log.h"

#include <stdio.h>
#include <time.h>
#include <varargs.h>
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif /* HAVE_SYS_FILE_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "defines.h"
#include "compat.h"
#include "client_msgs.h"
#include "variables.h"

char *
l_strerror(errnum)
  int   errnum;
{
  return(strerror(errnum));
}

/* log_error: prints a error message to the console. This is indented
   for server side errors only. */
/* FIXME: should have a compile time option to go to stderr, error
   file, or syslog */
void
log_error(va_alist)
  va_dcl
{
  va_list   list;
  char      *format;
  char      err_buf[MAX_LINE];
  
  va_start(list);
  format = va_arg(list, char *);
  vsprintf(err_buf, format, list);
  va_end(list);

  if (get_verbosity())
  {
    fprintf(stderr, "error: %s\n", err_buf);
  }
}

/* FIXME: should modify this routine to log to syslog as per compile
   time option */
/* log query information */
int
log_entry(va_alist)
  va_dcl
{
  va_list   ap;
  FILE      *fp;
  char      *format;
  char      *filename;
  char      *hostname;
  int       fd;

  va_start(ap);
  filename  = va_arg (ap, char*);
  format    = va_arg(ap, char*);

  if ((fp = fopen(filename, "a")) == NULL)
  {
    /* we cannot attempt to write to the log file here, because we
       just failed to open it. */
    fprintf(stderr, "ERROR save_into_log: could not open file %s", filename);
    print_error(UNIDENT_ERROR, "");

    return FALSE;
  }

  fd = fileno(fp);

  /* lock for exclusive use  - default is to wait */
  if (sys_file_lock(fd, FILE_LOCK) != 0)
  {
    log_entry(RWHOIS_LOG_FILE,
              "ERROR save_into_log: could not start lock %s", 
              filename);
    print_error(UNIDENT_ERROR,"");
    return(FALSE);
  }

  timestamp(fp);
  hostname = get_client_hostname(1);  /* fileno of stdout is the client sock */
  fprintf(fp, " [%-15s] PID: %-8d", hostname, (int) getpid());
  vfprintf(fp, format, ap);
  fprintf(fp, "\n");
  if (sys_file_lock(fd, FILE_UNLOCK) != 0) {
    log_entry(RWHOIS_LOG_FILE,
              "ERROR save_into_log: could not release lock %s", 
              filename);
    return (FALSE);
  }

  fclose(fp);
  return (TRUE);
}

/* gets hostname of the *client* */
char *
get_client_hostname(sock)
  int sock;             /* peer socket */
{
  static char           buf[MAX_LINE];
  static int            tried_once = FALSE;
  struct sockaddr_in    name;
  int                   namelen = sizeof(name);

  if (tried_once)
  {
    return (buf);
  }
  tried_once = TRUE;

  if (getpeername(sock, (struct sockaddr *)&name, &namelen) != 0)
  {
    print_error(FAILED_PEERNAME, "error: %s", l_strerror(errno));
    sprintf(buf, "<FAILED>");
  }
  else
  {
    sprintf(buf, "%s", inet_ntoa(name.sin_addr) );
  }
  
  return (buf);
}

/**************************************************************************
    logs time of query
**************************************************************************/
void
timestamp(f)
  FILE *f;
{
  time_t        now;
  struct tm     *ts;
  static char   *months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
  };

  now = time((time_t*)NULL);              /* current time */
  ts = localtime(&now);           /* get now in pieces */
  fprintf(f, "%2d-%s-%02d %02d:%02d:%02d", ts->tm_mday, months[ts->tm_mon],
          ts->tm_year % 100, ts->tm_hour, ts->tm_min, ts->tm_sec);
}
