/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "strutil.h"

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "defines.h"


/* strutil.c: a collection of string manipulation tools. */
/* History: originally located in misc.c */


/* stripchar: Strips out all occurences of a specific character (in
   place) */
char *
stripchar(str, ch)
  char *str;
  char ch;
{
  int i;
  int j;

  for (i = 0, j = 0; i < strlen(str); i++)
  {
    if (str[i] != ch)
    {
      str[j++] = str[i];
    }
  }
  str[j] = '\0';

  return(str);
}

/* strip_trailing: strips any characters matching ch off of the end of
   a string. */
char *
strip_trailing(str, ch)
  char *str;
  char ch;
{
  int len = strlen(str);
  int i;

  for (i = len - 1; str[i] == ch; i--)
  {
    str[i] = '\0';
  }

  return(str);
}

/* strip_leading: strips any characters matching ch off the front of a
   string. */
char *
strip_leading(str, ch)
  char *str;
  char ch;
{
  char *p1;
  char *p2;

  p1 = p2 = str;
  while (*p1 == ch)
  {
    p1++;
  }

  /* copy to front only if necessary */
  if (p1 != p2)
  {
    while (*p2)
    {
      *p2++ = *p1++;
    }
  }
  return(str);
}

char *
strip_control(str)
  char *str;
{
  int i;
  int j;

  for (i = 0, j = 0; i < strlen(str); i++)
  {
    if (!iscntrl(str[i]))
    {
      str[j++] = str[i];
    }
  }
  str[j] = '\0';

  return(str);
}

/* rtrim: strips whitespace off of the end of a string. */
char *
rtrim(str)
  char *str;
{
  int i;
  
  for (i = strlen(str) - 1; isspace(str[i]); i--)
  {
    str[i] = '\0';
  }

  return(str);
}

/* ltrim: strips the whitespace off the the front of a string */
char *
ltrim(str)
  char *str;
{
  char *p1;
  char *p2;

  p1 = p2 = str;
  while (isspace(*p1))
  {
    p1++;
  }

  /* copy to front only if necessary */
  if (p1 != p2)
  {
    while (*p2)
    {
      *p2++ = *p1++;
    }
  }
  return(str);
}

/* trim: trim whitespace off of both ends of a string (in place) */
char *
trim(str)
  char *str;
{
  rtrim(str);
  ltrim(str);

  return(str);
}


/***************************************************************************
 reverses the string
****************************************************************************/
char *
strrev(str)
  char *str;
{
  char *head;
  char *tail;
  char c;

  for (head = str, tail = str + strlen(str) - 1; head < tail; 
       head++, tail--)
  {
    c     = *head;
    *head = *tail;
    *tail = c;
  }
  return(str);
}
