/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "mkdb.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "defines.h"
#include "hitinfo.h"
#include "parse.h"
#include "misc.h"
#include "stack.h"
#include "search.h"
#include "boolean.h"
#include "log.h"

/* local prototypes */

static void mkdb_set_search PROTO((char *query, int *search_type,
                                   int *comparison));

/*************************************************************************
  run_query
**************************************************************************/
mkdb_hitinfo_struct *
mkdb_run_query(query, maxhits, error_code)
  char  *query;
  int   maxhits;
  int   *error_code;

{
  char                      *cp = query;
  char                      *next;
  char                      word[MAX_LINE];
  char                      words[MAX_LINE];
  int                       search_type;
  int                       comparison;
  mkdb_hitinfo_struct       *hits  = NULL;
  mkdb_hitinfo_struct       *hits1 = NULL;
  mkdb_hitinfo_struct       *hits2 = NULL;
  stack_struct              *stack = NULL;
  logical_separator_type    operator;

  /****
    first get all the queries done
    ****/

  *words ='\0';
  
  while (*cp)
  {
    next = get_word (cp, word);

    if (is_operator(word) && next != NULL && strlen (words))
    {
      mkdb_set_search(words, &search_type, &comparison);
      hits = search_index(0, words, search_type, comparison, maxhits,
                          error_code);
      stack = push_stack(stack, hits);
      *words = '\0';
    }
    else
    {
      if (*words)
      {
        strcat(words, " ");
      }
      strcat(words, word);
    }
    cp = next;
  }
  if (*words) {
    mkdb_set_search(words, &search_type, &comparison);
    hits = search_index(0, words, search_type, comparison, maxhits,
                        error_code);
    stack = push_stack (stack, hits);
  }

  /***
    then do the set stuff to bring together all the values
    ***/

  cp = query;
  while (*cp)
  {
    next = get_word(cp, word);
    if ((operator = is_operator(word)) && next != NULL)
    {
      switch (operator)
      {
      case LOGICAL_AND:
        hits1 = stack_state (stack); stack = pop_stack (stack);
        hits2 = stack_state (stack); stack = pop_stack (stack);
        hits = Intersection (hits1, hits2);
        stack = push_stack (stack, hits);
        break;
      case LOGICAL_OR:
        hits1 = stack_state (stack); stack = pop_stack (stack);
        hits2 = stack_state (stack); stack = pop_stack (stack);
        hits = Union (hits1, hits2);
        stack = push_stack (stack, hits); 
        break;
      default:
        break;
      }
    }
    cp = next; 
  }

  if (stack == NULL)
  {
    log_error ("mkdb_run_query: should not be null!\n");
    return (NULL);
  }

  hits = stack_state (stack);
  if ((stack = pop_stack (stack)) != NULL)
  {
    log_error("mkdb_run_query: should have been null!\n");
  }

  return (hits);
}

#define PARTWORD_WILDCARDS ".*"
/*************************************************************************
  sets the search mechanism
**************************************************************************/
static void
mkdb_set_search(query, search_type, comparison)
  char  *query;
  int   *search_type;
  int   *comparison;
{
  int last_char;

  if ((last_char = strlen(query)-1) > 0 && 
      strchr (PARTWORD_WILDCARDS, query[last_char]) != NULL)
  {
    query[last_char] = '\0';
    *comparison = PARTIAL_COMPARE;
  }
  else
  {
    *comparison = FULL_COMPARE;
  }

  *search_type = BINARY_SEARCH;
}

/*************************************************************************
  closes the index file
**************************************************************************/
void
mkdb_close_index()
{
  char  *words          = NULL;
  int   search_type     = 0;
  int   comparison      = 0;
  int   maxhits         = 0;
  int   *error_code     = NULL;

  search_index(1, words, search_type, comparison, maxhits,
               error_code);
}
