/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
/**************************************************************************
  Change history

9/13/94 Included additional col in binary_search, etc to mention that
        element deleted (TRUE) or not (FALSE).
***************************************************************************/
#include "search.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "client_msgs.h"
#include "defines.h"
#include "fileinfo.h"
#include "llist.h"
#include "log.h"
#include "misc.h"
#include "mkdb.h"
#include "strutil.h"

static int MKabort = 0;

/********
  local prototypes
********/

static mkdb_hitinfo_struct *binary_search PROTO((FILE *fp, long size,
                                                 char *search_value,
                                                 int comparison, int maxhits,
                                                 int *error_code,
                                                 mkdb_hitinfo_struct *hits));

static int backup PROTO((FILE *fp, int low, int *offset, int high));

static mkdb_hitinfo_struct *full_scan PROTO((FILE *fp, char *search_value,
                                             int comparison, int maxhits,
                                             int *error_code,
                                             mkdb_hitinfo_struct *hits));

static int compare_strings PROTO((char *search_value, char *curr_value,
                                  int comparison_type));

static int strCMP PROTO((char *a, char *b));

static int strnCMP PROTO((char *a, char *b, int n));


/********
  local structures
********/

typedef struct _mkdb_index_info
{
  int   size;
  FILE  *fp;
} mkdb_index_info;

#define NEW_MKDB_INDEX_INFO (mkdb_index_info *) calloc ((unsigned)1,\
  (unsigned)sizeof(mkdb_index_info))

/**************************************************************************
  Function: This set of procedures searches the index. Returns a linked
  list of fileno/offsets

  Returns: NULL if none found, otherwise ptrs.

  By: Mark Kosters

  Modified by Mark kosters to allow for multiple indexes on 12/1/94
**************************************************************************/
mkdb_hitinfo_struct *
search_index (close_index, search_value, search_type, 
              comparison, maxhits, error_code)
  int   close_index;
  char  *search_value;
  int   search_type;
  int   comparison;
  int   maxhits;
  int   *error_code;
{
  mkdb_fileinfo_struct  *fileinfo;
  static llist_struct   *open_index_list = NULL;
  static llist_struct   *cur;
  mkdb_index_info       *index_info;
  mkdb_hitinfo_struct   *hits = NULL;
  int                   index_no;

  if (!close_index)
  {
    /* if the index list is NULL, open the index files required */
    if (open_index_list == NULL)
    {
      index_no = 0;

      while ((fileinfo = get_fileinfo (ENUM_INDEX_FILE, index_no)) != NULL)
      {
        index_info = NEW_MKDB_INDEX_INFO;
        index_info->size = fileinfo->size;
        index_no = fileinfo->index_no+1;
        if ((index_info->fp = fopen (fileinfo->filename, "r")) == NULL)
        {
          print_error(MISC_DIAG,
                      "search_index: can not open index file |%s|\n",
                      fileinfo->filename);
          return NULL;
        }
        open_index_list = append_llist(open_index_list, index_info);
      }
    }
  }
  else
  {
    /* for closing the indexes, cleaning up */
    cur = open_index_list;
    while (cur != NULL)
    {
      index_info = cur->datum;
      fclose (index_info->fp);
      free (index_info);
      cur = cur->nxt;
    }
    free_llist (open_index_list);
    open_index_list = NULL;
  }
  cur = open_index_list;
  while (cur != NULL)
  {
    index_info = cur->datum;
    switch (search_type)
    {
    case FULL_SCAN: 
      fseek (index_info->fp, 0, 0);
      hits = full_scan(index_info->fp, search_value, comparison, 
                       maxhits, error_code, hits); 
      break;
    default:
      hits = binary_search(index_info->fp, index_info->size, 
                           search_value, comparison, maxhits,
                           error_code, hits);
      break;
    }
    cur = cur->nxt;
  }
  return (hits);
}

/**************************************************************************
  Function: This procedure searches the index. Returns a linked
  list of fileno/offsets

  Returns: NULL if none found, otherwise ptrs.

  By: Mark Kosters
**************************************************************************/
static mkdb_hitinfo_struct *
binary_search(fp, size, search_value, comparison,
              maxhits, error_code, hits)
  FILE                  *fp;
  long                  size;
  char                  *search_value;
  int                   comparison;
  int                   maxhits;
  int                   *error_code;
  mkdb_hitinfo_struct   *hits;
{
  char  buf[MAX_BUF];
  char  ndxvalue[MAX_BUF];
  long  file_offset;
  long  file_no;
  long  deleted;
  int   go;
  int   y;
  int   low = 0;
  int   savemid = 0;
  int   bottom  = 0;
  int   found   = -1;
  int   mid;
  int   high    = size;
  int   reccnt  = 0;


  while ( low < high  && !MKabort)
  {
    mid = (low + high) / 2;
    if (backup(fp, low, &mid, high) == -1 || savemid == mid || mid == high)
    {
      break;
    }

    savemid = mid;

    if (fseek (fp, mid, 0) != 0 || fgets(buf, MAX_BUF, fp) == NULL)
    {
      printf ("Error: search_index on seek/gets\n");
      printf ("got buff %s\n", buf);
      fseek (fp, 0, 2);
      printf ("max size is %ld\n", ftell(fp));
      print_error (MISC_DIAG, "search_index find: error on gets in seek\n");
      return (NULL);
    }
    mkdb_break_out(buf, ndxvalue, &file_offset, (int *)&file_no,
                   (int *)&deleted);
    y = compare_strings (search_value, ndxvalue, comparison);
    if (y < 0 )
    {
      high= mid - 1;
    }
    else if (y > 0)
    {
      low = mid + 1;
    }
    else
    {
      if (found == -1)
      {
        bottom = high;
      }
      found = mid;
      high = mid - 1;
    }  
  }

  if (found == -1)
  {
    return (hits);
  }
  else
  {
    high = bottom;
    mid = found;
  }

  fseek (fp, mid, 0);

  /* Now proceed to go forward */
  go = TRUE;
  while (ftell (fp) < high  && go && !MKabort)
  {
    if (fgets(buf, MAX_BUF, fp) == NULL)
    {
      fprintf (stderr, "searching for %d end %d\n", mid, high);
      print_error (MISC_DIAG,
                   "search_index lin_search : error on gets in seek");
      return (NULL);
    }
    mkdb_break_out(buf, ndxvalue, &file_offset, (int *)&file_no,
                   (int *)&deleted);
    y = compare_strings (search_value, ndxvalue, comparison);
    if (y == 0)
    {
      if (reccnt++ >= maxhits)
      {
        *error_code = MKDB_OVER_MAXHIT_LIMIT;
        go = FALSE;
      }
      else
      {
        hits = store_hits (hits, file_no, file_offset, deleted);
      }
    }
    else
    {
      go = FALSE;
    }
  }

  return (hits);
}

/**************************************************************************
  Function: Separates the value from the offset/fileno/value pairs
  Returns TRUE if okay
  FALSE otherwise

  By: Mark Kosters
  **************************************************************************/
int
mkdb_break_out(buf, ndxvalue, offset, file_no, deleted)
  char *buf;
  char *ndxvalue;
  long *offset;
  int  *file_no;
  int  *deleted;


{
  char tag[MAX_BUF];
  char datum[MAX_BUF];

  if (parse_line (buf, tag, datum))
  {
    *offset = atol (tag);
    strcpy (buf, datum);
    if (parse_line (buf, tag, datum))
    {
      *file_no = atoi (tag);
      strcpy (buf, datum);
      if (parse_line (buf, tag, datum))
      {
        *deleted = atoi (tag);
        strcpy (ndxvalue, datum);
        ltrim (ndxvalue);
        return (TRUE);
      }
    }
  }

  return (FALSE);
}

/****************************************************************************
 * backup --
 *  search backwards for newline - if none found
 *  then reverse direction and start over
*****************************************************************************/
static int
backup(fp, low, offset, high)
  FILE          *fp;
  register int  low;
  register int  *offset;
  register int  high;
{
  register int  c;
  register int  inc = -1;
  int           save = *offset;

  if (*offset == 0)
  {
    return(*offset);
  }

  for (; *offset < high && fseek(fp, *offset, 0) == 0; *offset += inc)
  {
    c = fgetc(fp);
    if (c == '\n' || *offset == 0)
    {
      return (*offset == 0) ? *offset : (*offset)++;
    }
    else if (c == EOF)      /* should never happen */
    {
      break;
    }
    if (*offset <= low)   /* start over, reversing direction */
    {
      *offset = save;
      inc = 1;
    }
  }

  return -1L;
}

/****************************************************************************
  Function: full_scan runs a linear scan on the index
************************************************************************/
static mkdb_hitinfo_struct *
full_scan(fp, search_value, comparison, maxhits,
          error_code, hits)
  FILE                  *fp;
  char                  *search_value;
  int                   comparison;
  int                   maxhits;
  int                   *error_code;
  mkdb_hitinfo_struct   *hits;

{
  char  buf[MAX_BUF];
  char  ndxvalue[MAX_BUF];
  long  file_offset, mid;
  int   go = 1;
  int   y;
  int   file_no;
  int   deleted;
  int   reccnt=0;

  mid = ftell (fp);
  
  while (fgets(buf, MAX_BUF, fp) != NULL && go && !MKabort)
  {
    mkdb_break_out(buf, ndxvalue, &file_offset, &file_no, &deleted);

    y = compare_strings (search_value, ndxvalue, comparison);

    if (y == 0)
    {
      if (reccnt++ >= maxhits)
      {
        *error_code = MKDB_OVER_MAXHIT_LIMIT;
        go = FALSE;
      }
      else
      {
        hits = store_hits (hits, file_no, file_offset, deleted);
      }
    }
    else if (y < 0)
    {
      fseek (fp, mid, 0);
      go = 0;
    }
  }

  /* this will be a linked list return */
  return (hits);
}


/****************************************************************************
  Function: comparses val1 and val2 - returns the result based on
    comparison type
************************************************************************/
static int
compare_strings (search_value, curr_value, comparison_type)
  char  *search_value;
  char  *curr_value;
  int   comparison_type;
{
  switch (comparison_type)
  {
  case FULL_COMPARE:
    return (strCMP(search_value,  curr_value));
    break;
  case PARTIAL_COMPARE:
    return (strnCMP(search_value,  curr_value, strlen(search_value)));
    break;
  case SUBSTR_COMPARE:
    return(strcasecmp(curr_value, search_value));
    break;
  default:
    log_error("invalid search type %d", comparison_type);
    return(-1);
  }
}

/* strCMP - created by Bob McCollum since SRI did not provide it */
static int
strCMP(a, b)
  char *a;
  char *b;
{
  int i;

  for (i = 0 ; toupper(a[i]) == toupper(b[i]); i++)
  {
    if (a[i] == '\0')
    {
      return (0);
    }
  }

  return ( toupper(a[i]) - toupper(b[i]) );
}

/* strnCMP - created by Mark Kosters since SRI did not provide it */
static int
strnCMP(a, b, n)
  char  *a;
  char  *b;
  int   n;
{
  int i;

  for (i = 0; (toupper(a[i]) == toupper(b[i])) &&  (i < n); i++)
  {
    if (a[i] == '\0')
    {
      return (0);
    }
  }
  if (i == n )
  {
    return (0);
  }

  return (toupper(a[i]) - toupper(b[i]));
}
