/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "display.h"

#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "defines.h"
#include "misc.h"
#include "client_msgs.h"
#include "client_info.h"
#include "display_conf.h"
#include "holdconnect.h"

/****************************************************************************
  calls set or display based on passed value
****************************************************************************/
int
set_or_display_display(str)
  char *str;
{
  char  word[MAX_LINE];
  char  *cp;
  char  *next;
  int   retval  = 0;

  cp   = str;
  next = get_word (cp, word);

  if (!*str)
  {
    retval = display_display(return_all_defined_displays());
  }
  else if (!*next)
  {
    retval = set_display(str, return_all_defined_displays());
  }
  else if (STR_EQ(word, "activate"))
  {
    retval = set_display(next, return_all_defined_displays());
  }
  else if (STR_EQ(word, "capable"))
  {
    set_client_capabilities(next);
  }
  else
  {
    print_error (INVALID_DIRECTIVE_PARAM, "");
    return FALSE;
  }
  if (!retval)
  {
    print_error(UNAUTH_DISPLAY, "");
    return FALSE;
  }

  print_ok();
  return TRUE;
}

/****************************************************************************
  sets the display type
****************************************************************************/
int
set_display(str, cur)
  char                  *str;
  output_command_struct *cur;

{
  char *cp, *next, word[MAX_LINE];
  int retval = FALSE;

  cp = str;
  next = get_word (cp, word);
  if (is_valid_display_command(word, cur)) {
    if (strlen (next))
      retval = set_display_status (next, word);
    else 
      retval = set_display_status ("all", word);
  }
  return (retval);
}

/****************************************************************************
 returns TRUE if valid display command
         FALSE if not
****************************************************************************/
int
is_valid_display_command(str, cur)
  char                  *str;
  output_command_struct *cur;
{
  int length = strlen (str);

  while (cur != NULL) {
    if (cur->command != NULL && length >= cur->command_len && 
        STRN_EQ(str, cur->command, cur->command_len))
    {
      return (TRUE);
    }

    cur = cur->nxt;
  }
  
  return (FALSE);
}

/****************************************************************************
  shows the display type  
****************************************************************************/
int
display_display(cur)
  output_command_struct *cur;

{
  int cnt = 0;

  while (cur != NULL)
  {
    if (cur->command != NULL)
    {
      if (cnt++)
      {
        printf ("%%display\n");
      }
      printf ("%%display name: %s\n", cur->command);
      printf ("%%display length: %d\n", cur->command_len);
      printf ("%%display description: %s\n", cur->description);
    }
    cur = cur->nxt;
  }

  return TRUE;
}
