/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "main.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "conf.h"
#include "daemon.h"
#include "defines.h"
#include "initialize.h"
#include "log.h"
#include "read_config.h"
#include "security.h"
#include "session.h"
#include "variables.h"

/* local prototypes */

static int usage PROTO((char *prog_name));

int
main(argc, argv)
  int  argc;
  char *argv[];
{
  extern char   *optarg;
  int           badopts         = FALSE;
  char          *config_file    = NULL;
  char          c;

  /* set initial configuration data values */
  init_config_data();
  
  /* unbuffer stdout */
  /* FIXME: this is probably redundant -- runsession does a setvbuf */
  setbuf(stdout, NULL);

  /* parse command line options */
  while ((c = getopt(argc, argv, "c:rsdiv")) != EOF) {
    switch (c) {
    case 'c':
      config_file = optarg;
      break;
    case 'r':
      set_root_server(TRUE);
      break;
    case 's':
      set_chrooted(FALSE);
      break;
    case 'd':
      set_server_type(DAEMON_SERVER);
      break;
    case 'i':
      set_server_type(INETD_SERVER);
      break;
    case 'v':
      set_verbosity(2);
      break;
    default:
      badopts = TRUE;
      break;
    }
  }
  
  if (badopts) {
    usage(argv[0]);
  }

  /* default section */

  if (config_file == NULL)
  {
    config_file = DEFAULT_RWHOIS_CONFIG_FILE;
  }
  /* we are only setting this so we can pass it as a parameter to
     rwhoisparse */
   set_config_file(config_file);
  
  /* do 1-time initialization */

  initialize_environment();
  
  if (!read_all_config_files(config_file, FALSE))
  {
    exit (1);
  }

  chdir_root_dir();
  
  if (get_verbosity() > 1)
  {
    display_config_data(stderr);
  }
  
  /* do security stuff -- chroot() or just change id, or nothing */
  if (!setup_security())
  {
    log_error("Security setup failed! Exiting.");
    exit(1);
  }

  if (is_daemon_server())
  {
    run_daemon();
  }
  else
  {
    run_session();
  }

  exit_program();

  /* this is just to suppress warnings */
  return(0);
}

/* usage: prints the usage statement */
static int
usage(prog_name)
  char *prog_name;
{
  fprintf(stderr,
          "usage: %s [-c config_file] [-r] [-s] [-v] [-di]\n", prog_name);
  fprintf(stderr,
          "       -c config_file: location of the base configuration file\n");
  fprintf(stderr,
          "       -r: root server; do not generate punt referrals\n");
  fprintf(stderr,
          "       -s: do not chroot\n");
  fprintf(stderr,
          "       -v: verbose\n");
  fprintf(stderr,
          "       -d: run as daemon (standalone)\n");
  fprintf(stderr,
          "       -i: run as single shot (inetd)\n");
exit(64);
}






