/* insert std header here */

#include "session.h"

#include <stdio.h>
#include <string.h>

#include "daemon.h"
#include "deadman.h"
#include "defines.h"
#include "misc.h"
#include "strutil.h"
#include "variables.h"
#include "whois.h"

/* print_welcome_header: prints the standard rwhois banner greeting */
static void
print_welcome_header()
{
  char *hostname;

  hostname = get_local_hostname();
  
  printf("%%RWhois V-%s:", RWHOIS_PROTOCOL_VERSION);
  
  if (hostname && *hostname)
  {
    printf(" %s", hostname);
  }
  else
  {
    printf(" UNKNOWN");
  }

  printf(" (by InterNIC Registration Services V-%s)\n",
         RWHOIS_SERVER_VERSION);

  fflush(stdout);
}

/*************************************************************************
  exits the program
**************************************************************************/
void
exit_program()
{
  /* clean up if necessary */
  if (is_daemon_server())
  {
    close_main_socket();
  }

  exit(0);
}

/* run_session: runs a rwhois client session */
void
run_session()
{
  char  target[MAX_LINE];
  int   root_srv = is_root_server();
  
  print_welcome_header();

  /* set the input to line buffering, not block buffering */
  setlinebuf(stdin);

  /* unbuffer the output */
#ifdef SETVBUF_REVERSED
  setvbuf(stdout, _IONBF, (char *)NULL, 0);
#else
  setvbuf(stdout, (char *)NULL, _IONBF, 0);
#endif
  
  do {
    set_timer(get_deadman_time(), is_a_deadman);

    readline(target, MAX_LINE);
    
    unset_timer();
  } while (whois(target, root_srv, FALSE));
}
