/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/

#ifndef _IO_H_
#define _IO_H_

/* includes */

#include "common.h"

#include <stdio.h>
#include "defines.h"
#include "fileinfo.h"

/* types */

typedef struct _rwhois_io_struct
{
  char  object_name[MAX_LINE];
  int   type;
  FILE  *base_fp;
  FILE  *index_fp;
  char  index_unsorted[MAX_FILENAME];
  char  index_sorted[MAX_FILENAME];
  int   file_no;
  int   num_recs;
  int   total_recs;
  int   mode;
  char  data_file[MAX_FILENAME];
} rwhois_io_struct;

#define NEW_RWHOIS_IO \
   (rwhois_io_struct *) calloc ((unsigned)1,\
                                (unsigned)sizeof(rwhois_io_struct))

/* defines */

/* modes for the particular files */
#define INITIALIZE      1
#define APPEND          2
#define AUTHORITATIVE   0
#define CACHED          1

/* prototypes */

void set_ensure_in_jail PROTO((int flag));

int  get_ensure_in_jail PROTO(());

FILE *get_object_db PROTO((char *object_name, char *data_file, int mode,
                           int type, int *file_no));

int  initialize_rwhois_db PROTO((char *base_name, rwhois_io_struct *ptr));

int  append_rwhois_db PROTO((char *base_name, rwhois_io_struct *ptr));

int  next_file_no_from_fileinfo PROTO((mkdb_fileinfo_struct *fileinfo));

int  next_index_no_from_fileinfo PROTO((mkdb_fileinfo_struct *fileinfo));

void clear_existing_indexes PROTO((char *basefname));

void save_in_db PROTO((FILE *fp, char *data_file, char *type, int file_no,
                       int num_recs));

void increment_record_count PROTO((char *object_name, char *data_file));

int  flush_and_sort PROTO((int in_jail));

int  sort_index PROTO((int in_jail, char *unsorted_name, char *sorted_name));


#endif /* _IO_H_ */
